@@ -6,8 +6,21 @@
 
 from __future__ import absolute_import
 
-from .pythia import Pythia2PDGIDBiMap
-from .geant import Geant2PDGIDBiMap
+from typing import Tuple
 
-from .evtgen import EvtGenName2PDGIDBiMap
-from .evtgen import PDG2EvtGenNameMap, EvtGen2PDGNameMap
+from .evtgen import EvtGen2PDGNameMap, EvtGenName2PDGIDBiMap, PDG2EvtGenNameMap
+from .geant import Geant2PDGIDBiMap
+from .pythia import Pythia2PDGIDBiMap
+
+__all__ = (
+    "EvtGen2PDGNameMap",
+    "EvtGenName2PDGIDBiMap",
+    "PDG2EvtGenNameMap",
+    "Geant2PDGIDBiMap",
+    "Pythia2PDGIDBiMap",
+)
+
+
+def __dir__():
+    # type: () -> Tuple[str, ...]
+    return __all__
