@@ -18,14 +18,13 @@ from __future__ import absolute_import
 import csv
 
 from .. import data
-from ..pdgid import PDGID
 from ..exceptions import MatchingIDNotFound
+from ..pdgid import PDGID
 
-
-with data.open_text(data, "pdgid_to_geant3id.csv") as _f:
+with data.basepath.joinpath("pdgid_to_geant3id.csv").open() as _f:
     _bimap = {
         int(v["GEANT3ID"]): int(v["PDGID"])
-        for v in csv.DictReader(l for l in _f if not l.startswith("#"))
+        for v in csv.DictReader(line for line in _f if not line.startswith("#"))
     }
 
 
@@ -57,7 +56,7 @@ class Geant3ID(int):
             if v == pdgid:
                 return cls(k)
         raise MatchingIDNotFound(
-            "Non-existent Geant3ID for input PDGID {0} !".format(pdgid)
+            "Non-existent Geant3ID for input PDGID {} !".format(pdgid)
         )
 
     def to_pdgid(self):
