@@ -9,7 +9,7 @@ This is a conversion file, not part of the public API.
 
 The default CSV files can be updated directly using the command:
 
-    >>> python -m particle.particle.convert regenerate 2020 <version_number>    # doctest: +SKIP
+    >>> python -m particle.particle.convert regenerate 2021 <version_number>    # doctest: +SKIP
 
 A custom fwf file and LaTeX file can be converted into the CSV format using:
 
@@ -17,7 +17,7 @@ A custom fwf file and LaTeX file can be converted into the CSV format using:
 
 The command is typically
 
-    >>> python -m particle.particle.convert extended output.csv particle/data/mass_width_2008.fwf <version_number>    # doctest: +SKIP
+    >>> python -m particle.particle.convert extended output.csv src/particle/data/mass_width_2008.fwf <version_number>    # doctest: +SKIP
 
 This file requires pandas. But most users will not need this file, as it only
 converts PDG data files into the CSV file(s) the public API tools use. The tests
@@ -34,7 +34,7 @@ combined with one or more LaTeX files describing the pair (PDG ID, LaTeX name):
 
 You can also read in a modern "standard" file (this will produce fewer columns):
 
-    >>> ext_table = get_from_pdg_mcd('particle/data/mass_width_2020.mcd')
+    >>> ext_table = get_from_pdg_mcd('particle/data/mass_width_2021.mcd')
 
 A utility is even provided to use the modern table to update the full table:
 
@@ -53,11 +53,21 @@ When you are done, you can save one or more of the tables:
 
 import os
 from datetime import date
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    Optional,
+    TextIO,
+    Tuple,
+    TypeVar,
+    Union,
+)
+
 import numpy as np
 import pandas as pd
 
-from typing import TextIO, List, Optional, Dict, TypeVar, Callable, Union, Iterable, Any
-
 try:
     from io import StringIO
 except ImportError:  # Python2 workaround, could also use six
@@ -66,26 +76,33 @@ except ImportError:  # Python2 workaround, could also use six
     except ImportError:
         from StringIO import StringIO  # type: ignore
 
-from ..pdgid import PDGID
-from ..pdgid import is_baryon
-
+from .. import data
+from ..pdgid import PDGID, is_baryon
 from .enums import (
-    SpinType,
-    Parity,
     Charge,
-    Inv,
-    Status,
-    Parity_mapping,
-    Inv_mapping,
-    Status_mapping,
     Charge_mapping,
+    Inv,
+    Inv_mapping,
+    Parity,
+    Parity_mapping,
+    Status_mapping,
 )
 
-from .. import data
-
-
 FILE_DIR = os.path.dirname(os.path.realpath(__file__))
 
+__all__ = (
+    "convert",
+    "get_from_pdg_extended",
+    "get_from_pdg_mcd",
+    "get_from_latex",
+    "update_from_mcd",
+)
+
+
+def __dir__():
+    # type: () -> Tuple[str, ...]
+    return __all__
+
 
 def get_from_latex(filename):
     # type: (str) -> pd.Series
@@ -228,12 +245,12 @@ def get_from_pdg_extended(filename, latexes=()):
     pdg_table_inv.Latex.where(
         ~full_inversion,
         pdg_table_inv.Latex.str.replace(
-            r"^(\\mathrm{|)([a-zA-Z\\][a-zA-Z]*)", r"\1\\overline{\2}"
+            r"^(\\mathrm{|)([a-zA-Z\\][a-zA-Z]*)", r"\1\\overline{\2}", regex=True
         ),
         inplace=True,
     )
     pdg_table_inv.Latex = (
-        pdg_table_inv.Latex.str.replace(r"+", r"%")
+        pdg_table_inv.Latex.str.replace(r"+", r"%", regex=True)
         .str.replace(r"-", r"+")
         .str.replace(r"%", r"-")
     )
@@ -272,11 +289,11 @@ def sort_particles(table):
 def get_from_pdg_mcd(filename):
     # type: (str) -> pd.DataFrame
     """
-    Reads in a current-style PDG .mcd file (mass_width_2020.mcd file tested).
+    Reads in a current-style PDG .mcd file (mass_width_2021.mcd file tested).
 
     Example
     -------
-    >>> mcd_table = get_from_pdg_mcd('particle/data/mass_width_2020.mcd')
+    >>> mcd_table = get_from_pdg_mcd('particle/data/mass_width_2021.mcd')
     """
 
     # The format here includes the space before a column
@@ -318,9 +335,22 @@ def get_from_pdg_mcd(filename):
             ),
         )
 
+        # Check for duplicates!
+        duplicated_ids = (
+            nar.duplicated(subset=["ID1"], keep=False) & nar["ID1"].notna()
+            | nar.duplicated(subset=["ID2"], keep=False) & nar["ID2"].notna()
+            | nar.duplicated(subset=["ID3"], keep=False) & nar["ID3"].notna()
+            | nar.duplicated(subset=["ID4"], keep=False) & nar["ID4"].notna()
+        )
+        if nar[duplicated_ids].shape[0] > 0:
+            print("DUPLICATES:\n", nar[duplicated_ids])
+        assert (
+            nar[duplicated_ids].shape[0] == 0
+        ), "Duplicate entries found in {} !".format(filename)
+
     ds_list = []
     for i in range(4):
-        name = "ID{0}".format(i + 1)
+        name = "ID{}".format(i + 1)
         d = nar[~pd.isna(nar[name])].copy()
         d["ID"] = d[name].astype(int)
         nc = d.NameCharge.str.split(expand=True)
@@ -349,7 +379,7 @@ def update_from_mcd(full_table, update_table):
 
     Example
     -------
-    >>> new_table = update_from_mcd('mass_width_2008.fwf', 'mass_width_2020.mcd')    # doctest: +SKIP
+    >>> new_table = update_from_mcd('mass_width_2008.fwf', 'mass_width_2021.mcd')    # doctest: +SKIP
     """
 
     full_table = full_table.copy()
@@ -361,12 +391,12 @@ def update_from_mcd(full_table, update_table):
     return full_table
 
 
-def produce_files(particle2008, particle2020, version, year):
+def produce_files(particle2008, particle2021, version, year):
     # type: (str, str, str, str) -> None
     "This produces listed output files from all input files."
 
-    with data.open_text(data, "mass_width_2008.fwf") as fwf_f:
-        with data.open_text(data, "pdgid_to_latexname.csv") as csv_f:
+    with data.basepath.joinpath("mass_width_2008.fwf").open() as fwf_f:
+        with data.basepath.joinpath("pdgid_to_latexname.csv").open() as csv_f:
             full_table = get_from_pdg_extended(fwf_f, [csv_f])
 
     # Entries to remove, see comments in file mass_width_2008_ext.fwf:
@@ -382,11 +412,11 @@ def produce_files(particle2008, particle2020, version, year):
     # f.write(version_header(particle2008, version))
     # full_table.to_csv(f, float_format="%.12g")
 
-    with data.open_text(data, "mass_width_" + year + ".mcd") as mcd_f:
+    with data.basepath.joinpath("mass_width_" + year + ".mcd").open() as mcd_f:
         ext_table = get_from_pdg_mcd(mcd_f)
 
-    with data.open_text(data, "mass_width_2008_ext.fwf") as fwf_f:
-        with data.open_text(data, "pdgid_to_latexname.csv") as csv_f:
+    with data.basepath.joinpath("mass_width_2008_ext.fwf").open() as fwf_f:
+        with data.basepath.joinpath("pdgid_to_latexname.csv").open() as csv_f:
             addons = get_from_pdg_extended(fwf_f, [csv_f])
 
     # Only keep rows present in the .mcd file specified by year
@@ -403,9 +433,9 @@ def produce_files(particle2008, particle2020, version, year):
 
     new_table = update_from_mcd(full_table, ext_table)
 
-    particle2020 = str(particle2020)  # Conversion to handle pathlib on Python < 3.6
-    with open(particle2020, "w", newline="\n", encoding="utf-8") as f:
-        f.write(version_header(particle2020, version))
+    particle2021 = str(particle2021)  # Conversion to handle pathlib on Python < 3.6
+    with open(particle2021, "w", newline="\n", encoding="utf-8") as f:
+        f.write(version_header(particle2021, version))
         new_table.to_csv(f, float_format="%.12g")
 
 
@@ -432,7 +462,7 @@ def main(version, year):
 
 def convert(version, output, fwf, latex=None):
     # type: (str, str, str, Optional[str]) -> None
-    latexes = [data.open_text(data, "pdgid_to_latexname.csv")]
+    latexes = [data.basepath / "pdgid_to_latexname.csv"]
     if latex:
         latexes.append(latex)
     table = get_from_pdg_extended(fwf, latexes)
