@@ -9,50 +9,51 @@ from __future__ import absolute_import, division, print_function
 # Python standard library
 import csv
 from copy import copy
-
 from functools import total_ordering
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    SupportsInt,
+    TextIO,
+    Tuple,
+    Union,
+)
 
 # External dependencies
 import attr
-
-from typing import (
-    Optional,
-    Any,
-    Dict,
-    Tuple,
-    List,
-    Callable,
-    Iterable,
-    Iterator,
-    SupportsInt,
-    Union,
-    TextIO,
-    Set,
-)
-
+from deprecated import deprecated
 from hepunits.constants import c_light
 
 from .. import data
-from ..pdgid import PDGID
-from ..pdgid import is_valid
-from ..pdgid.functions import _digit
-from ..pdgid.functions import Location
-from .regex import getname, getdec
-from .enums import (
-    SpinType,
-    Parity,
-    Charge,
-    Inv,
-    Status,
-    Parity_undo,
-    Parity_prog,
-    Charge_undo,
-    Charge_prog,
-    Charge_mapping,
-)
-from .utilities import programmatic_name, str_with_unc, latex_to_html_name
-from .kinematics import width_to_lifetime
 from ..converters.evtgen import EvtGenName2PDGIDBiMap
+from ..pdgid import PDGID, is_valid
+from ..pdgid.functions import Location, _digit
+from ..typing import HasOpen, HasRead
+from .enums import (
+    Charge,
+    Charge_mapping,
+    Charge_undo,
+    Inv,
+    Parity,
+    Parity_undo,
+    SpinType,
+    Status,
+)
+from .kinematics import width_to_lifetime
+from .regex import getname
+from .utilities import latex_to_html_name, programmatic_name, str_with_unc
+
+__all__ = ("Particle", "ParticleNotFound", "InvalidParticle")
+
+
+def __dir__():
+    # type: () -> Tuple[str, ...]
+    return __all__
 
 
 class ParticleNotFound(RuntimeError):
@@ -211,7 +212,9 @@ class Particle(object):
         minus_one, converter=_none_or_positive_converter
     )  # type: Optional[float]
     _three_charge = attr.ib(Charge.u, converter=Charge)  # charge * 3
-    I = attr.ib(none_float, converter=_isospin_converter)  # type: Optional[float]
+    I = attr.ib(  # noqa: E741
+        none_float, converter=_isospin_converter
+    )  # type: Optional[float]
     # J = attr.ib(None)  # Total angular momentum
     G = attr.ib(Parity.u, converter=Parity)  # Parity: '', +, -, or ?
     P = attr.ib(Parity.u, converter=Parity)  # Space parity
@@ -231,7 +234,7 @@ class Particle(object):
         )
 
     # Loaded table of entries
-    _table = None  # type: Optional[Set[Particle]]
+    _table = None  # type: Optional[List[Particle]]
 
     # Names of loaded tables
     _table_names = None  # type: Optional[List[str]]
@@ -268,7 +271,7 @@ class Particle(object):
 
     @classmethod
     def all(cls):
-        # type: () -> Set[Particle]
+        # type: () -> List[Particle]
         """
         Access, hence get hold of, the internal particle data CSV table,
         loading it from the default location if no table has yet been loaded.
@@ -277,7 +280,7 @@ class Particle(object):
         if not cls.table_loaded():
             cls.load_table()
 
-        return cls._table if cls._table is not None else set()
+        return cls._table if cls._table is not None else []
 
     @classmethod
     def to_list(
@@ -405,7 +408,7 @@ class Particle(object):
             for fld in exclude_fields:
                 try:
                     tbl_names.remove(fld)
-                except:
+                except ValueError:
                     pass
 
         # Start with the full table
@@ -553,7 +556,7 @@ class Particle(object):
         if append and not cls.table_loaded():
             cls.load_table(append=False)  # default load
         elif not append:
-            cls._table = set()
+            cls._table = []
             cls._table_names = []
 
         # Tell MyPy that this is true
@@ -561,25 +564,26 @@ class Particle(object):
         assert cls._table_names is not None
 
         if filename is None:
-            with data.open_text(data, "particle2020.csv") as f:
-                cls.load_table(f, append=append, _name="particle2020.csv")
-            with data.open_text(data, "nuclei2020.csv") as f:
+            with data.basepath.joinpath("particle2021.csv").open() as f:
+                cls.load_table(f, append=append, _name="particle2021.csv")
+            with data.basepath.joinpath("nuclei2020.csv").open() as f:
                 cls.load_table(f, append=True, _name="nuclei2020.csv")
             return
-        elif not hasattr(filename, "read"):
-            cls._table_names.append(str(filename))
-            # Conversion to handle pathlib on Python < 3.6:
-            open_file = open(str(filename))
-        else:
-            assert not isinstance(filename, str)  # Tell typing that this is true
-            tmp_name = _name or getattr(filename, "name")
+        elif isinstance(filename, HasRead):
+            tmp_name = _name or filename.name
             cls._table_names.append(
-                tmp_name or "{0!r} {1}".format(filename, len(cls._table_names))
+                tmp_name or "{!r} {}".format(filename, len(cls._table_names))
             )
             open_file = filename
+        elif isinstance(filename, HasOpen):
+            cls._table_names.append(str(filename))
+            open_file = filename.open()
+        else:
+            cls._table_names.append(str(filename))
+            open_file = open(filename)
 
         with open_file as f:
-            r = csv.DictReader(l for l in f if not l.startswith("#"))
+            r = csv.DictReader(line for line in f if not line.startswith("#"))
 
             for v in r:
                 try:
@@ -590,7 +594,7 @@ class Particle(object):
                     if value in cls._table:
                         cls._table.remove(value)  # type: ignore
 
-                    cls._table.add(
+                    cls._table.append(
                         cls(
                             pdgid=value,
                             mass=float(v["Mass"]),
@@ -615,6 +619,8 @@ class Particle(object):
                 except ValueError:
                     pass
 
+        cls._table = sorted(cls._table)
+
     # The following __le__ and __eq__ needed for total ordering (sort, etc)
 
     def __lt__(self, other):
@@ -946,7 +952,7 @@ class Particle(object):
         if self.mass is None:
             return "None"
         else:
-            return "{0} MeV".format(
+            return "{} MeV".format(
                 str_with_unc(self.mass, self.mass_upper, self.mass_lower)
             )
 
@@ -976,8 +982,8 @@ C (charge parity) = {C:<6}  I (isospin)       = {self.I!s:<7}  G (G-parity)
             val += "    SpinType: {self.spin_type!s}\n".format(self=self)
         if self.quarks:
             val += "    Quarks: {self.quarks}\n".format(self=self)
-        val += "    Antiparticle name: {iself.name} (antiparticle status: {self.anti_flag.name})".format(
-            iself=self.invert(), self=self
+        val += "    Antiparticle name: {inv_self.name} (antiparticle status: {self.anti_flag.name})".format(
+            inv_self=self.invert(), self=self
         )
         return val
 
@@ -1020,69 +1026,63 @@ C (charge parity) = {C:<6}  I (isospin)       = {self.I!s:<7}  G (G-parity)
             If no matching PDG ID is found in the loaded data table(s).
         """
         if not is_valid(value):
-            raise InvalidParticle("Input PDGID {0} is invalid!".format(value))
+            raise InvalidParticle("Input PDGID {} is invalid!".format(value))
 
         for item in cls.all():
             if item.pdgid == value:
                 return item
         else:
-            raise ParticleNotFound("Could not find PDGID {0}".format(value))
+            raise ParticleNotFound("Could not find PDGID {}".format(value))
 
     @classmethod
-    def findall(
+    def finditer(
         cls,
         filter_fn=None,  # type: Optional[Callable[[Particle], bool]]
         particle=None,  # type: Optional[bool]
         **search_terms  # type: Any
     ):
-        # type: (...) -> List[Particle]
+        # type: (...) -> Iterator[Particle]
         """
-        Search for a particle, returning a list of candidates.
+        Search for a particle, returning an iterator of candidates.
 
-        The first and only positional argument is given each particle
+        The first and only positional argument is given to each particle
         candidate, and returns True/False. Example:
 
-            >>> Particle.findall(lambda p: 'p' in p.name)    # doctest: +SKIP
-            # Returns list of all particles with p somewhere in name
+            >>> Particle.finditer(lambda p: 'p' in p.name)    # doctest: +SKIP
+            # Returns iterator of all particles with p somewhere in name
 
         You can pass particle=True/False to force a particle or antiparticle.
         If this is not callable, it will do a "fuzzy" search on the name. So this is identical:
 
-            >>> Particle.findall('p')    # doctest: +SKIP
-            # Returns list of all particles with p somewhere in name (same as example above)
+            >>> Particle.finditer('p')    # doctest: +SKIP
+            # Returns literator of all particles with p somewhere in name (same as example above)
 
         You can also pass keyword arguments, which are either called with the
         matching property if they are callable, or are compared if they are not.
         This would do an exact search on the name, instead of a fuzzy search:
 
            >>> # Returns proton and antiproton only
-           >>> Particle.findall(pdg_name='p')    # doctest: +NORMALIZE_WHITESPACE
-           [<Particle: name="p", pdgid=2212, mass=938.272081 ± 0.000006 MeV>,
-            <Particle: name="p~", pdgid=-2212, mass=938.272081 ± 0.000006 MeV>,
-            <Particle: name="p", pdgid=1000010010, mass=938.272081 ± 0.000006 MeV>,
-            <Particle: name="p~", pdgid=-1000010010, mass=938.272081 ± 0.000006 MeV>]
+           >>> Particle.finditer(pdg_name='p')    # doctest: +SKIP
+           # Returns iterator
 
            >>> # Returns proton only
-           >>> Particle.findall(pdg_name='p', particle=True)    # doctest: +NORMALIZE_WHITESPACE
-           [<Particle: name="p", pdgid=2212, mass=938.272081 ± 0.000006 MeV>,
-           <Particle: name="p", pdgid=1000010010, mass=938.272081 ± 0.000006 MeV>]
+           >>> Particle.finditer(pdg_name='p', particle=True)    # doctest: +SKIP
+           # Returns iterator
 
         Versatile searches require a (lambda) function as argument:
 
         >>> # Get all neutral beauty hadrons
-        >>> Particle.findall(lambda p: p.pdgid.has_bottom and p.charge==0)    # doctest: +SKIP
+        >>> Particle.finditer(lambda p: p.pdgid.has_bottom and p.charge==0)    # doctest: +SKIP
         >>>
         >>> # Trivially find all pseudoscalar charm mesons
-        >>> Particle.findall(lambda p: p.pdgid.is_meson and p.pdgid.has_charm and p.spin_type==SpinType.PseudoScalar)  # doctest: +SKIP
+        >>> Particle.finditer(lambda p: p.pdgid.is_meson and p.pdgid.has_charm and p.spin_type==SpinType.PseudoScalar)  # doctest: +SKIP
 
-        See also ``find``, which throws an exception if the particle is not found or too many are found.
+        See also ``findall``, which returns the same thing, but as a list.
         """
 
         # Note that particle can be called by position to keep compatibility with Python 2, but that behavior should
         # not be used and will be removed when support for Python 2.7 is dropped.
 
-        results = set()
-
         # Filter out values
         for item in cls.all():
             # At this point, continue if a match fails
@@ -1130,12 +1130,68 @@ C (charge parity) = {C:<6}  I (isospin)       = {self.I!s:<7}  G (G-parity)
 
             # If the loop was not broken
             else:
-                results.add(item)
-
-        # Matches are sorted so the top one is "best"
-        return sorted(results)
+                yield item
 
     @classmethod
+    def findall(
+        cls,
+        filter_fn=None,  # type: Optional[Callable[[Particle], bool]]
+        particle=None,  # type: Optional[bool]
+        **search_terms  # type: Any
+    ):
+        # type: (...) -> List[Particle]
+
+        """
+        Search for a particle, returning a list of candidates.
+
+        The first and only positional argument is given to each particle
+        candidate, and returns True/False. Example:
+
+            >>> Particle.findall(lambda p: 'p' in p.name)    # doctest: +SKIP
+            # Returns list of all particles with p somewhere in name
+
+        You can pass particle=True/False to force a particle or antiparticle.
+        If this is not callable, it will do a "fuzzy" search on the name. So this is identical:
+
+            >>> Particle.findall('p')    # doctest: +SKIP
+            # Returns list of all particles with p somewhere in name (same as example above)
+
+        You can also pass keyword arguments, which are either called with the
+        matching property if they are callable, or are compared if they are not.
+        This would do an exact search on the name, instead of a fuzzy search:
+
+           >>> # Returns proton and antiproton only
+           >>> Particle.findall(pdg_name='p')    # doctest: +NORMALIZE_WHITESPACE
+           [<Particle: name="p", pdgid=2212, mass=938.272081 ± 0.000006 MeV>,
+            <Particle: name="p~", pdgid=-2212, mass=938.272081 ± 0.000006 MeV>,
+            <Particle: name="p", pdgid=1000010010, mass=938.272081 ± 0.000006 MeV>,
+            <Particle: name="p~", pdgid=-1000010010, mass=938.272081 ± 0.000006 MeV>]
+
+           >>> # Returns proton only
+           >>> Particle.findall(pdg_name='p', particle=True)    # doctest: +NORMALIZE_WHITESPACE
+           [<Particle: name="p", pdgid=2212, mass=938.272081 ± 0.000006 MeV>,
+           <Particle: name="p", pdgid=1000010010, mass=938.272081 ± 0.000006 MeV>]
+
+        Versatile searches require a (lambda) function as argument:
+
+        >>> # Get all neutral beauty hadrons
+        >>> Particle.findall(lambda p: p.pdgid.has_bottom and p.charge==0)    # doctest: +SKIP
+        >>>
+        >>> # Trivially find all pseudoscalar charm mesons
+        >>> Particle.findall(lambda p: p.pdgid.is_meson and p.pdgid.has_charm and p.spin_type==SpinType.PseudoScalar)  # doctest: +SKIP
+
+        See also ``finditer``, which provides an iterator instead of a list.
+        """
+
+        return list(
+            cls.finditer(filter_fn=filter_fn, particle=particle, **search_terms)
+        )
+
+    @classmethod
+    @deprecated(
+        version="0.16.0",
+        reason="This method is deprecated and will be removed from version 0.17.0. Use findall instead.",
+    )
     def find(cls, *args, **search_terms):
         # type: (Any, Any) -> Particle
         """
@@ -1186,7 +1242,7 @@ C (charge parity) = {C:<6}  I (isospin)       = {self.I!s:<7}  G (G-parity)
         if matches:
             return matches[0]
         else:
-            raise ParticleNotFound("{0} not found in particle table".format(name))
+            raise ParticleNotFound("{} not found in particle table".format(name))
 
     @classmethod
     def from_string_list(cls, name):
@@ -1264,7 +1320,7 @@ C (charge parity) = {C:<6}  I (isospin)       = {self.I!s:<7}  G (G-parity)
 
         if not vals:
             raise ParticleNotFound(
-                "Could not find particle {0} or {1}".format(maxname, name)
+                "Could not find particle {} or {}".format(maxname, name)
             )
 
         if len(vals) > 1 and mat["mass"] is not None:
